/*
 * parse.h: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#ifndef _PARSE_H
#define _PARSE_H

#include <vdr/tools.h>
#include <vdr/epg.h>
#include <libxml/parser.h>
#include <time.h>

#include "maps.h"

class cXMLTVEvent
{
private:
    char *title;
    char *shorttext;
    char *description;
    time_t starttime;
    int duration;
    time_t vps;
    tEventID eventid;
#if VDRVERSNUM >= 10711
    uchar parentalRating;
    uchar contents[MaxEventContents];
#endif
public:
    cXMLTVEvent();
    ~cXMLTVEvent();
    void Clear();
    void SetTitle(const char *Title);
    void SetShortText(const char *ShortText);
    void SetDescription(const char *Description);
    void SetStartTime(time_t StartTime)
    {
        starttime=StartTime;
    }
    void SetDuration(int Duration)
    {
        duration=Duration;
    }
    void SetVps(time_t Vps)
    {
        vps=Vps;
    }
    void SetEventID(tEventID EventID)
    {
        eventid=EventID;
    }
    time_t Vps(void) const
    {
        return vps;
    }
    int Duration()
    {
        return duration;
    }
    time_t StartTime()
    {
        return starttime;
    }
    const char *Title(void) const
    {
        return title;
    }
    const char *ShortText(void) const
    {
        return shorttext;
    }
    const char *Description(void) const
    {
        return description;
    }
    tEventID EventID(void) const
    {
        return eventid;
    }
};

class cParse
{
private:
    char *name;
    cEPGMappings *maps;
    cXMLTVEvent xevent;
    cCharSetConv *conv;
    u_long DoSum(u_long sum, const char *buf, int nBytes);
    cEvent *SearchEvent(cSchedule* schedule, cXMLTVEvent *xevent);
    time_t ConvertXMLTVTime2UnixTime(char *xmltvtime);
    bool FetchEvent(xmlNodePtr node);
    cEPGMapping *EPGMapping(const char *ChannelName);
    bool PutEvent(cSchedule* schedule,cEvent *event,cXMLTVEvent *xevent, cEPGMapping *map);
public:
    cParse(const char *Name, cEPGMappings *Maps);
    ~cParse();
    bool Process(char *buffer, int bufsize, int days);
};

#endif
