/*
 * setup.cpp: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#include "setup.h"

#include <vdr/osdbase.h>

#define CHNUMWIDTH (numdigits(Channels.MaxNumber())+1)

char *strcatrealloc(char *dest, const char *src)
{
    if (!src || !*src)
        return dest;

    size_t l = (dest ? strlen(dest) : 0) + strlen(src) + 1;
    if (dest)
    {
        dest = (char *)realloc(dest, l);
        strcat(dest, src);
    }
    else
    {
        dest = (char*)malloc(l);
        strcpy(dest, src);
    }
    return dest;
}

cMenuSetupXmltv2vdr::cMenuSetupXmltv2vdr(cPluginXmltv2vdr *Plugin)
{
    baseplugin=Plugin;
    sourcesBegin=sourcesEnd=mappingBegin=mappingEnd=0;
    Output();
}

void cMenuSetupXmltv2vdr::Output(void)
{
    if (!baseplugin) return;
    int current=Current();
    Clear();
    cOsdItem *first=newtitle(tr("epg sources"));
    Add(first,true);

    if (!baseplugin->EPGSourceCount())
    {
        baseplugin->ReadInEPGSources();
        if (!baseplugin->EPGSourceCount())
        {
            Add(new cOsdItem(tr("no epgsources installed"),osUnknown,false));
            Display();
            return;
        }
    }
    sourcesBegin=Current()+1;
    for (int i=0; i<baseplugin->EPGSourceCount(); i++)
    {
        cEPGSource *epgsrc=baseplugin->EPGSource(i);
        if (epgsrc)
        {
            Add(new cOsdItem(epgsrc->Name()),true);
        }
    }
    sourcesEnd=Current();

    Add(newtitle(tr("epg source channels")),true);
    generatesumchannellist();
    mappingBegin=Current()+1;
    for (int i=0; i<channels.Size(); i++)
    {
        bool mapped=false;
        cEPGMapping *map=baseplugin->EPGMapping(channels[i]);
        if (map)
        {
            if (map->NumChannelIDs()) mapped=true;
        }
        cString buffer = cString::sprintf ("%s:\t%s",channels[i],mapped ? tr("mapped") : "");
        Add (new cOsdItem (buffer),true);
    }
    mappingEnd=Current();

    if (current==-1)
    {
        SetCurrent(first);
        CursorDown();
    }
    else
    {
        SetCurrent(Get(current));
    }
    Display();
}

void cMenuSetupXmltv2vdr::generatesumchannellist()
{
    cStringList oldchannels;
    for (int i=0; i<channels.Size(); i++)
    {
        oldchannels.Append(strdup(channels[i]));
    }

    channels.Clear();
    for (int i=0; i<baseplugin->EPGSourceCount(); i++)
    {
        cEPGSource *epgsrc=baseplugin->EPGSource(i);
        if (epgsrc)
        {
            cStringList *channellist=epgsrc->ChannelList();
            for (int x=0; x<(*channellist).Size(); x++)
            {
                char *txt=strdup((*channellist)[x]);
                if (txt)
                {
                    char *a=strchr(txt,'*');
                    if (a)
                    {
                        *a=0;
                        bool found=false;
                        for (int t=0; t<channels.Size(); t++)
                        {
                            if (!strcmp(channels[t],txt)) found=true;
                        }
                        if (!found)
                        {
                            channels.Append(txt);
                        }
                        else
                        {
                            free(txt);
                        }
                    }
                }
            }
        }
    }
    channels.Sort();

    for (int i=0; i<oldchannels.Size(); i++)
    {
        if (channels.Find(oldchannels[i])==-1)
        {
            cEPGMapping *map=baseplugin->EPGMapping(oldchannels[i]);
            if (map)
            {
                map->ReplaceChannels(0,NULL);
                map->ChangeFlags(0);
                char *name=NULL;
                if (asprintf(&name,"channel.%s",map->ChannelName())!=-1)
                {
                    SetupStore(name,"0;");
                    free(name);
                }
            }
        }
    }
}

cOsdItem *cMenuSetupXmltv2vdr::newtitle(const char *s)
{
    cString buffer = cString::sprintf("---- %s ----", s);
    return new cOsdItem (buffer,osUnknown,false);
}

void cMenuSetupXmltv2vdr::Store(void)
{
}

eOSState cMenuSetupXmltv2vdr::edit()
{
    if ((Current()>=sourcesBegin) && (Current()<=sourcesEnd))
    {
        return AddSubMenu(new cMenuSetupXmltv2vdrChannel(baseplugin,this,Current()-sourcesBegin));
    }

    if ((Current()>=mappingBegin) && (Current()<=mappingEnd))
    {
        return AddSubMenu(new cMenuSetupXmltv2vdrChannelMap(baseplugin,this,Current()-mappingBegin));
    }
    return osUnknown;
}

eOSState cMenuSetupXmltv2vdr::ProcessKey (eKeys Key)
{
    eOSState state = cOsdMenu::ProcessKey(Key);
    if (HasSubMenu()) return osContinue;
    switch (state)
    {
    case osUnknown:
        if (Key==kOk)
        {
            state=edit();
            if (state==osUnknown)
            {
                Store();
                state=osBack;
            }
        }
        break;

    default:
        break;
    }
    return state;
}

// --------------------------------------------------------------------------------------------------------

cMenuSetupXmltv2vdrChannel::cMenuSetupXmltv2vdrChannel(cPluginXmltv2vdr *Plugin, cMenuSetupXmltv2vdr *Menu, int Index)
{
    menu=Menu;
    baseplugin=Plugin;
    sel=NULL;
    days=0;
    channellist=NULL;

    epgsrc=baseplugin->EPGSource(Index);
    if (!epgsrc) return;

    SetSection(cString::sprintf("%s '%s' : %s",trVDR("Plugin"), baseplugin->Name(), epgsrc->Name()));

    Add(newtitle(tr("options")));
    days=epgsrc->DaysInAdvance();
    Add(new cMenuEditIntItem(tr("days in advance"),&days,1,epgsrc->DaysMax()));

    Add(newtitle(tr("channels provided")));

    channellist=epgsrc->ChannelList();
    if (!channellist) return;

    sel=new int[(*channellist).Size()];
    if (!sel) return;

    for (int i=0; i<(*channellist).Size(); i++)
    {
        char *txt=strdup((*channellist)[i]);
        if (txt)
        {
            char *a=strchr(txt,'*');
            if (a)
            {
                *a=0;
                sel[i]=1;
            }
            else
            {
                sel[i]=0;
            }
            Add(new cMenuEditBoolItem(txt,&sel[i],"",tr("selected")));
            free(txt);
        }
    }
}

cMenuSetupXmltv2vdrChannel::~cMenuSetupXmltv2vdrChannel()
{
    if (sel) delete [] sel;
    if (menu) menu->Output();
}

cOsdItem *cMenuSetupXmltv2vdrChannel::newtitle(const char *s)
{
    cString buffer = cString::sprintf("---- %s ----", s);
    return new cOsdItem (buffer,osUnknown,false);
}

void cMenuSetupXmltv2vdrChannel::Store(void)
{
    if ((!channellist) || (!sel) || (!epgsrc)) return;

    char *fname1=NULL;
    char *fname2=NULL;
    if (asprintf(&fname1,"%s/%s",EPGSOURCES,epgsrc->Name())==-1) return;
    if (asprintf(&fname2,"%s/%s.new",EPGSOURCES,epgsrc->Name())==-1)
    {
        free(fname1);
        return;
    }

    FILE *r=fopen(fname1,"r+");
    if (!r)
    {
        free(fname1);
        free(fname2);
        return;
    }
    int oldmask=umask(0664);
    FILE *w=fopen(fname2,"w+");
    umask(oldmask);
    if (!w)
    {
        fclose(w);
        unlink(fname2);
        free(fname1);
        free(fname2);
        return;
    }

    char *line=NULL;
    size_t lsize;
    int linenr=1;
    while (getline(&line,&lsize,r)!=-1)
    {
        if (linenr>2)
        {
            for (int i=0; i<(*channellist).Size(); i++)
            {
                char *txt=strdup((*channellist)[i]);
                if (txt)
                {
                    char *ptxt=txt;
                    char *a=strchr(txt,'*');
                    if (a) *a=0;
                    ptxt=line;
                    if (ptxt[0]=='*') ptxt++;

                    if (!strncmp(ptxt,txt,strlen(txt)))
                    {
                        if (sel[i])
                        {
                            fprintf(w,"*%s",ptxt);
                        }
                        else
                        {
                            fprintf(w,"%s",ptxt);
                        }
                    }
                    free(txt);
                }
            }
        }
        else
        {
            fprintf(w,"%s",line);
        }
        linenr++;
    }
    if (line) free(line);
    struct stat statbuf;
    if (fstat(fileno(r),&statbuf)!=-1)
    {
        if (fchown(fileno(w),statbuf.st_uid,statbuf.st_gid)) {};
        if (fchmod(fileno(w),statbuf.st_mode | S_IRGRP | S_IWGRP)) {};
    }
    fclose(w);
    fclose(r);
    rename(fname2,fname1);
    free(fname1);
    free(fname2);

    baseplugin->ReadInEPGSources(true);
}

// --------------------------------------------------------------------------------------------------------

cMenuSetupXmltv2vdrChannelMap::cMenuSetupXmltv2vdrChannelMap(cPluginXmltv2vdr *Plugin, cMenuSetupXmltv2vdr *Menu, int Index)
{
    baseplugin=Plugin;
    menu=Menu;
    flags=0;
    if (Index>menu->ChannelList()->Size()) return;
    channel=(*menu->ChannelList())[Index];
    if (!channel) return;

    SetPlugin(baseplugin);

    cEPGMapping *oldmap=baseplugin->EPGMapping(channel);
    if (oldmap)
    {
        map=new cEPGMapping(*oldmap);
    }
    else
    {
        map=new cEPGMapping(channel,NULL);
    }
    if (!map) return;

    title=cString::sprintf("%s - %s '%s' : %s",trVDR("Setup"),trVDR("Plugin"), baseplugin->Name(), channel);
    SetTitle(title);

    SetHelp(tr("unmap"),tr("map"));
    flags=map->Flags();
    output();
}

cMenuSetupXmltv2vdrChannelMap::~cMenuSetupXmltv2vdrChannelMap()
{
    if (map) delete map;
    if (menu) menu->Output();
}

cOsdItem *cMenuSetupXmltv2vdrChannelMap::option(const char *s, bool yesno)
{
    cString buffer = cString::sprintf("%s:\t%s", s, yesno ? trVDR("yes") : trVDR("no"));
    return new cOsdItem (buffer,osUnknown,false);
}

cOsdItem *cMenuSetupXmltv2vdrChannelMap::newtitle(const char *s)
{
    cString buffer = cString::sprintf("---- %s ----", s);
    return new cOsdItem (buffer,osUnknown,false);
}

void cMenuSetupXmltv2vdrChannelMap::output(void)
{
    if (!map) return;

    int current=Current();

    Clear();

    Add(newtitle(tr("channel options")));

    Add(new cMenuEditBitItem(tr("append non existing events"),&flags,OPT_APPEND));

    if ((flags & OPT_APPEND)!=OPT_APPEND)
    {
        Add(new cMenuEditBitItem(tr("short text"),&flags,USE_SHORTTEXT));
        Add(new cMenuEditBitItem(tr("long text"),&flags,USE_LONGTEXT));
        if ((flags & USE_LONGTEXT)==USE_LONGTEXT)
        {
            Add(new cMenuEditBitItem(tr("merge long texts"),&flags,OPT_MERGELTEXT));
        }
        else
        {
            Add(option(tr("merge long texts"),false));
        }
    }
    else
    {
        flags|=USE_SHORTTEXT;
        flags|=USE_LONGTEXT;
        flags&=~OPT_MERGELTEXT;
        Add(option(tr("short text"),true));
        Add(option(tr("long text"),true));
        Add(option(tr("merge long texts"),false));
    }
#if 0
    Add(new cMenuEditBitItem(tr("country and date"),&flags,USE_COUNTRYDATE));
    Add(new cMenuEditBitItem(tr("original title"),&flags,USE_ORIGTITLE));
    Add(new cMenuEditBitItem(tr("category"),&flags,USE_CATEGORY));
    Add(new cMenuEditBitItem(tr("credits"),&flags,USE_CREDITS));
    Add(new cMenuEditBitItem(tr("rating"),&flags,USE_RATING));
    Add(new cMenuEditBitItem(tr("review"),&flags,USE_REVIEW));
    Add(new cMenuEditBitItem(tr("video"),&flags,USE_VIDEO));
    Add(new cMenuEditBitItem(tr("audio"),&flags,USE_AUDIO));
    Add(new cMenuEditBitItem(tr("vps"),&flags,OPT_VPS));
#endif

    Add(newtitle(tr("channel mappings")));
    for (int i=0; i<map->NumChannelIDs(); i++)
    {
        cChannel *chan=Channels.GetByChannelID(map->ChannelIDs()[i]);
        if (chan)
        {
            cString buffer = cString::sprintf("%-*i %s", CHNUMWIDTH, chan->Number(),chan->Name());
            Add(new cOsdItem(buffer));
        }
    }
    if (current!=-1) for (int i=1; i<=current; i++) CursorDown();

    Display();
}

eOSState cMenuSetupXmltv2vdrChannelMap::ProcessKey (eKeys Key)
{
    cOsdItem *item=NULL;
    eOSState state = cOsdMenu::ProcessKey(Key);
    if (HasSubMenu()) return osContinue;
    if (state==osContinue)
    {
        switch (Key)
        {
        case kLeft:
        case kRight:
            if ((Current()==1) || (Current()==3)) output();
            break;
        default:
            break;
        }
    }

    if (state==osUnknown)
    {
        switch (Key)
        {
        case kOk:
            Store();
            state=osBack;
            break;
        case kRed:
            item=Get(Current());
            if (item)
            {
                if (map)
                {
                    map->RemoveChannel(atoi(item->Text()));
                    output();
                }
            }
            break;
        case kGreen:
            return AddSubMenu(new cMenuSetupXmltv2vdrChannelsVDR(baseplugin,this,channel,title));
            break;
        default:
            break;
        }
    }
    return state;
}

void cMenuSetupXmltv2vdrChannelMap::AddChannel2Map(int ChannelNumber)
{
    if (map)
    {
        map->AddChannel(ChannelNumber);
        output();
    }
}

bool cMenuSetupXmltv2vdrChannelMap::EPGMappingExists(tChannelID ChannelID)
{
    if (!map) return true;
    for (int i=0; i<map->NumChannelIDs(); i++)
    {
        if (map->ChannelIDs()[i]==ChannelID) return true;
    }
    return false;
}

void cMenuSetupXmltv2vdrChannelMap::Store(void)
{
    if (!channel) return;
    char *name=NULL;
    if (asprintf(&name,"channel.%s",channel)==-1) return;

    char *value=NULL;
    if (asprintf(&value,"%i;",(int) flags)==-1)
    {
        free(name);
        return;
    }
    for (int i=0; i<map->NumChannelIDs(); i++)
    {
        cString ChannelID = map->ChannelIDs()[i].ToString();
        value=strcatrealloc(value,*ChannelID);
        if (i<map->NumChannelIDs()-1) value=strcatrealloc(value,";");
    }

    SetupStore(name,value);
    map->ChangeFlags(flags);
    baseplugin->EPGMappingReplace(map);

    free(name);
    free(value);
}

// --------------------------------------------------------------------------------------------------------

cMenuSetupXmltv2vdrChannelsVDR::cMenuSetupXmltv2vdrChannelsVDR(cPluginXmltv2vdr *Plugin,
        cMenuSetupXmltv2vdrChannelMap *Map, const char *Channel, cString Title)
        :cOsdMenu("",CHNUMWIDTH)
{
    baseplugin=Plugin;
    map=Map;
    SetHelp(NULL,NULL,tr("choose"));
    SetTitle(Title);

    for (cChannel *channel = Channels.First(); channel; channel=Channels.Next(channel))
    {
        if (!channel->GroupSep())
        {
            cString buf= cString::sprintf("%d\t%s",channel->Number(),channel->Name());
            if ((baseplugin->EPGMappingExists(channel->GetChannelID(),Channel)) || (map->EPGMappingExists(channel->GetChannelID())))
            {
                Add(new cOsdItem(buf,osUnknown,false));
            }
            else
            {
                Add(new cOsdItem(buf));
            }
        }
    }
}

eOSState cMenuSetupXmltv2vdrChannelsVDR::ProcessKey (eKeys Key)
{
    cOsdItem *item=NULL;
    eOSState state = cOsdMenu::ProcessKey(Key);
    if (HasSubMenu()) return osContinue;
    if (state==osUnknown)
    {
        switch (Key)
        {
        case kBack:
            state=osBack;
            break;
        case kYellow:
        case kOk:
            item=Get(Current());
            if (item)
            {
                if (map) map->AddChannel2Map(atoi(item->Text()));
            }
            state=osBack;
            break;
        default:
            break;
        }
    }
    return state;
}
