/*
 * xmltv2vdr.h: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#ifndef _XMLTV2VDR_H
#define _XMLTV2VDR_H

#define EPGSOURCES "/var/lib/epgsources"

#include <vdr/plugin.h>
#include "maps.h"
#include "parse.h"

static const char *VERSION        = "0.0.1";
static const char *DESCRIPTION    = trNOOP ( "Imports xmltv epg into vdr" );

class cEPGExecute : public cThread
{
private:
    const char *name;
    cParse *parse;
    bool ready2parse;
    bool pipe;
    int daysinadvance;
    int daysmax;
    bool ReadConfig();
    cStringList channels;
public:
    cEPGExecute(const char *Name,cEPGMappings *Maps);
    ~cEPGExecute();
    cStringList *GetChannelList()
    {
        return &channels;
    }
    int GetDaysMax()
    {
        return daysmax;
    }
    int GetDaysInAdvance()
    {
        return daysinadvance;
    }
    virtual void Action();
    void Stop()
    {
        Cancel(3);
    }
};

class cEPGSource : public cListObject
{
private:
    const char *name;
    cEPGExecute exec;
public:
    cEPGSource(const char *Name,cEPGMappings *Maps);
    ~cEPGSource();
    bool Execute();
    cStringList *ChannelList()
    {
        return exec.GetChannelList();
    }
    int DaysMax()
    {
        return exec.GetDaysMax();
    }
    int DaysInAdvance()
    {
        return exec.GetDaysInAdvance();
    }
    const char *Name()
    {
        return name;
    }
};

class cEPGSources : public cList<cEPGSource> {};


class cPluginXmltv2vdr : public cPlugin
{
private:
    cEPGMappings epgmappings;
    cEPGSources epgsources;
    void removeepgsources();
    void removeepgmappings();
    bool executeepgsources();
    bool epgsourceexists(const char *name);
public:
    void ReadInEPGSources(bool Reload=false);
    int EPGSourceCount()
    {
        return epgsources.Count();
    }
    cEPGSource *EPGSource(int Index)
    {
        return epgsources.Get(Index);
    }
    void EPGMappingReplace(cEPGMapping *newMapping);
    bool EPGMappingExists(tChannelID ChannelID, const char *Channel2Ignore=NULL);
    cEPGMapping *EPGMapping(const char *ChannelName);
    cPluginXmltv2vdr ( void );
    virtual ~cPluginXmltv2vdr();
    virtual const char *Version ( void )
    {
        return VERSION;
    }
    virtual const char *Description ( void )
    {
        return DESCRIPTION;
    }
    virtual const char *CommandLineHelp ( void );
    virtual bool ProcessArgs ( int argc, char *argv[] );
    virtual bool Initialize ( void );
    virtual bool Start ( void );
    virtual void Stop ( void );
    virtual void Housekeeping ( void );
    virtual void MainThreadHook ( void );
    virtual cString Active ( void );
    virtual time_t WakeupTime ( void );
    virtual const char *MainMenuEntry ( void );
    virtual cOsdObject *MainMenuAction ( void );
    virtual cMenuSetupPage *SetupMenu ( void );
    virtual bool SetupParse ( const char *Name, const char *Value );
    virtual bool Service ( const char *Id, void *Data = NULL );
    virtual const char **SVDRPHelpPages ( void );
    virtual cString SVDRPCommand ( const char *Command, const char *Option, int &ReplyCode );
};

#endif
